package net.yakiboo.yakito;

import java.io.File;
import java.util.HashMap;
import java.util.LinkedList;

import javax.swing.Icon;
import javax.swing.ImageIcon;

import net.yakiboo.yakito.io.YakiToFileSystem;
import net.yakiboo.yakito.io.YakiToOutput;
import net.yakiboo.yakito.io.YakiToXML;
import net.yakiboo.yakito.io.YakiToXMLSyntaxException;

/**
 * Clase que representa un perfil de conversion
 * 
 * @author Sergio Garcia Mondaray
 * 
 */
public class YakiToProfile {

	private String name = "";
	private String container = "";
	private String acodec = "";
	private int akbps;
	private int samplefreq;
	private int achannels;
	private int volume;
	private String vcodec = "";
	private int vkbps;
	private int fps;
	private String vtag = "";
	private int vsize, hsize;
	private String extraparams = "";
	private Icon image;

	private static LinkedList<YakiToProfile> profiles = null;

	/**
	 * Constructor de perfiles
	 * 
	 * @param name
	 *            Nombre del nuevo perfil
	 */
	public YakiToProfile(String name) {
		this.name = name;
	}

	/**
	 * Almacena el perfil actual en un archivo ".profile" de YakiTo
	 * 
	 * @param f
	 *            Archivo de destino
	 */
	public void saveProfileToFile(File f) {
		HashMap<String, String> map = new HashMap<String, String>();
		map.put("container", container);
		map.put("extraparams", extraparams);
		map.put("acodec", acodec);
		if (acodec.equals("") || acodec.equals("copy")) {
			// Audio deshabilitado
			map.put("akbps", "");
			map.put("samplefreq", "");
			map.put("achannels", "");
			map.put("volume", "");
		} else {
			// Audio habilitado
			String aux = "";
			if (akbps > 0)
				aux = akbps + "";
			map.put("akbps", aux);
			aux = "";
			if (samplefreq > 0)
				aux = samplefreq + "";
			map.put("samplefreq", aux);
			aux = "";
			if (achannels > 0)
				aux = achannels + "";
			map.put("achannels", aux);
			aux = "";
			if (volume != 0)
				aux = volume + "";
			map.put("volume", aux);
		}
		map.put("vcodec", vcodec);
		if (vcodec.equals("") || vcodec.equals("copy")) {
			map.put("vtag", "");
			map.put("vkbps", "");
			map.put("vfps", "");
			map.put("vsize", "");
			map.put("hsize", "");
		} else {
			String aux = "";
			if (vkbps > 0)
				aux = "" + vkbps;
			map.put("vkbps", aux);
			aux = "";
			if (fps > 0)
				aux = "" + fps;
			map.put("vfps", aux);
			aux = "";
			if (vsize > 0 && hsize > 0) {
				map.put("vsize", vsize + "");
				map.put("hsize", hsize + "");
			} else {
				map.put("vsize", "");
				map.put("hsize", "");
			}
			map.put("vtag", vtag);

		}
		map.put("extraparams", extraparams);
		YakiToXML.writeXMLFile(map, f);

	}

	/**
	 * Lista todos los perfiles disponibles en YakiTo
	 * 
	 * @return Lista de perfiles
	 */
	public static LinkedList<YakiToProfile> getAllProfiles() {
		if (profiles == null) {
			profiles = new LinkedList<YakiToProfile>();
			File parent = YakiToFileSystem.getProfilesLocation();
			File[] child = parent.listFiles();
			int cont = 0;
			for (File f : child) {
				if (f.getName().endsWith(".profile")) {
					try {
						HashMap<String, String> map = YakiToXML
								.parseSimpleXMLFile(f);
						YakiToProfile p = new YakiToProfile(f.getName()
								.replace(".profile", ""));

						String iconName = f.getName();
						iconName = iconName.replace(".profile", ".png");
						File auxF = new File(f.getParent()
								+ System.getProperty("file.separator")
								+ iconName);
						if (auxF.exists())
							p.setImage(new ImageIcon(auxF.getAbsolutePath()));
						String aux;
						aux = map.get("container");
						if (aux != null && aux.length() > 1)
							p.setContainer(aux);
						aux = map.get("acodec");
						if (aux != null && aux.length() > 1)
							p.setACodec(aux);
						aux = map.get("akbps");
						if (aux != null && aux.length() > 1)
							p.setAkbps(Integer.parseInt(aux));
						aux = map.get("samplefreq");
						if (aux != null && aux.length() > 1)
							p.setSamplefreq(Integer.parseInt(aux));
						aux = map.get("achannels");
						if (aux != null && aux.length() > 0)
							p.setAchannels(Integer.parseInt(aux));
						aux = map.get("vol");
						if (aux != null && aux.length() > 1)
							p.setVolume(Integer.parseInt(aux));
						aux = map.get("vcodec");
						if (aux != null && aux.length() > 1)
							p.setVCodec(aux);
						aux = map.get("vkbps");
						if (aux != null && aux.length() > 1)
							p.setVkbps(Integer.parseInt(aux));
						aux = map.get("vfps");
						if (aux != null && aux.length() > 1)
							p.setFps(Integer.parseInt(aux));
						aux = map.get("vtag");
						if (aux != null && aux.length() > 1)
							p.setVtag(aux);
						aux = map.get("hsize");
						if (aux != null && aux.length() > 1)
							p.setHsize(Integer.parseInt(aux));
						aux = map.get("vsize");
						if (aux != null && aux.length() > 1)
							p.setVsize(Integer.parseInt(aux));
						aux = map.get("extraparams");
						if (aux != null && aux.length() > 1)
							p.setExtraparams(aux);
						profiles.add(p);
						cont++;
					} catch (YakiToXMLSyntaxException ex) {
						YakiToOutput.error("El archivo de perfil \""
								+ f.getAbsolutePath()
								+ "\" contiene errores. Se ignorará.");
					}

				}
			}
			YakiToOutput
					.message("Parseando perfiles de conversión... Se han encontrado "
							+ cont + " perfiles.");
		}

		return profiles;
	}

	@Override
	public String toString() {
		return this.name;
	}

	/**
	 * Obtiene el nombre del perfil
	 * 
	 * @return Nombre del perfil
	 */
	public String getName() {
		return name;
	}

	/**
	 * Obtiene el contenedor multimedia utilizado por el perfil
	 * 
	 * @return Contenedor multimedia
	 */
	public String getContainer() {
		return container;
	}

	/**
	 * Establece el contenedor multimedia del perfil
	 * 
	 * @param container
	 *            Contenedor multimedia
	 */
	public void setContainer(String container) {
		this.container = container;
	}

	/**
	 * Obtiene el codec de audio utilizado por el perfil
	 * 
	 * @return Codec de audio
	 */
	public String getACodec() {
		return acodec;
	}

	/**
	 * Establece el codec de audio a utilizar
	 * 
	 * @param acodec
	 *            Codec de audio
	 */
	public void setACodec(String acodec) {
		this.acodec = acodec;
	}

	/**
	 * Obtiene el ratio de compresion de audio del perfil
	 * 
	 * @return Ratio de compresion (en kb/s)
	 */
	public int getAkbps() {
		return akbps;
	}

	/**
	 * Establece el ratio de compresion del audio del perfil
	 * 
	 * @param akbps
	 *            Ratio de compresion (en kb/s)
	 */
	public void setAkbps(int akbps) {
		this.akbps = akbps;
	}

	/**
	 * Obtiene la etiqueta de video del perfil
	 * 
	 * @return Etiqueta para el codec de video
	 */
	public String getVtag() {
		return vtag;
	}

	/**
	 * Establece la etiqueta del codec de video del perfil
	 * 
	 * @param vtag
	 *            Etiqueta de video
	 */
	public void setVtag(String vtag) {
		this.vtag = vtag;
	}

	/**
	 * Obtiene el codec de video utilizado por el perfil
	 * 
	 * @return Codec de video del perfil
	 */
	public String getVCodec() {
		return vcodec;
	}

	/**
	 * Establece el codec de video del perfil
	 * 
	 * @param vcodec
	 *            Codec de video
	 */
	public void setVCodec(String vcodec) {
		this.vcodec = vcodec;
	}

	/**
	 * Obtiene una cadena con los argumentos adicionales para FFMPEG
	 * 
	 * @return Parametros adicionales de conversion
	 */
	public String getExtraparams() {
		return extraparams;
	}

	/**
	 * Establece la cadena de argumentos adicionales de conversion
	 * 
	 * @param extraparams
	 *            Parametros adicionales de conversion
	 */
	public void setExtraparams(String extraparams) {
		this.extraparams = extraparams;
	}

	/**
	 * Obtiene la frecuencia de muestreo del audio
	 * 
	 * @return Frecuencia de muestreo actual
	 */
	public int getSamplefreq() {
		return samplefreq;
	}

	/**
	 * Establece la frecuencia de muestreo del audio
	 * 
	 * @param samplefreq
	 *            Frecuencia de muestreo nueva
	 */
	public void setSamplefreq(int samplefreq) {
		this.samplefreq = samplefreq;
	}

	/**
	 * Obtiene el numero de canales de audio
	 * 
	 * @return Numero de canales de audio
	 */
	public int getAchannels() {
		return achannels;
	}

	/**
	 * Establece el numero de canales de audio
	 * 
	 * @param achannels
	 *            Numero de canales de audio
	 */
	public void setAchannels(int achannels) {
		this.achannels = achannels;
	}

	/**
	 * Obtiene el volumen actual del audio
	 * 
	 * @return Volumen
	 */
	public int getVolume() {
		return volume;
	}

	/**
	 * Establece el volumen de audio actual
	 * 
	 * @param volume
	 *            Volumen del audio
	 */
	public void setVolume(int volume) {
		this.volume = volume;
	}

	/**
	 * Obtiene la resolucion vertical del video
	 * 
	 * @return Dimension vertical del video (en pixels)
	 */
	public int getVsize() {
		return vsize;
	}

	/**
	 * Establece la dimension vertical de video
	 * 
	 * @param vsize
	 *            Dimensione vertical
	 */
	public void setVsize(int vsize) {
		this.vsize = vsize;
	}

	/**
	 * Obtiene la dimension horizontal de video
	 * 
	 * @return Dimension horizontal (en pixels)
	 */
	public int getHsize() {
		return hsize;
	}

	/**
	 * Establece la dimension horizontal del video
	 * 
	 * @param hsize
	 *            Dimension horizontal (en pixels)
	 */
	public void setHsize(int hsize) {
		this.hsize = hsize;
	}

	/**
	 * Obtiene el ratio de compresion de video
	 * 
	 * @return Ratio de compresion de video (en kb/s)
	 */
	public int getVkbps() {
		return vkbps;
	}

	/**
	 * Establece el ratio de compresion de video
	 * 
	 * @param vkbps
	 *            Ratio de compresion nuevo (en kb/s)
	 */
	public void setVkbps(int vkbps) {
		this.vkbps = vkbps;
	}

	/**
	 * Obtiene el numero de capturas por segundo
	 * 
	 * @return Numero de imagenes por segundo
	 */
	public int getFps() {
		return fps;
	}

	/**
	 * Establece el numero de imagenes por segundo que componen el video
	 * 
	 * @param fps
	 *            Numero de imagenes por segundo
	 */
	public void setFps(int fps) {
		this.fps = fps;
	}

	/**
	 * Establece la imagen asociada al perfil
	 * 
	 * @param image
	 *            Imagen de perfil
	 */
	public void setImage(Icon image) {
		this.image = image;
	}

	/**
	 * Obtiene la imagen de perfil
	 * 
	 * @return Imagen del perfil
	 */
	public Icon getImage() {
		return image;
	}

	@Override
	public boolean equals(Object o) {
		boolean eq = o instanceof YakiToProfile;
		if (eq)
			eq = ((YakiToProfile) o).getName().equals(this.getName());
		return eq;
	}

}
