/**
 * YakiToUpdater.java
 */

package net.yakiboo.yakito.update;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipInputStream;

import net.yakiboo.yakito.config.YakiToConfig;
import net.yakiboo.yakito.io.YakiToFileSystem;
import net.yakiboo.yakito.io.YakiToOutput;
import net.yakiboo.yakito.utils.Stopwatch;

/**
 * Clase que gestiona las actualizaciones de YakiTo
 * 
 * @author Sergio Garcia Mondaray
 * 
 */
public class YakiToUpdater {

	/**
	 * Archivo de actualizaciones remoto
	 */
	private final String defaultPathFile = "http://www.yakiboo.net/yakito/updates/update.zip";

	YakiToConfig config = null;
	long lastUpdate = 0;

	public YakiToUpdater(YakiToConfig c) {
		config = c;
	}

	/**
	 * Comprueba si existen actualizaciones. Lo que hace es mirar si la fecha de
	 * modificacion del archivo de actualizaciones remoto es menor que la fecha
	 * de la ultima vez que se actualizo YakiTo.
	 * 
	 * @param lastUpdateKnown
	 *            Fecha de la ultima actualizacion realizada
	 * @return Si hay o no actualizaciones.
	 */
	public boolean checkUpdates() {
		long lastUpdateKnown = config.getLastKnownUpdate();
		boolean updates = false;
		try {
			YakiToOutput.message("Chequeando \"" + defaultPathFile
					+ "\" en busca de actualizaciones...");
			URL updatesURL = new URL(defaultPathFile);
			URLConnection connection = (HttpURLConnection) updatesURL
					.openConnection();

			lastUpdate = connection.getLastModified();
			if (lastUpdateKnown < lastUpdate) {
				updates = true;
				YakiToOutput.message("Existen actualizaciones nuevas.");
			} else {
				YakiToOutput.message("No existen actualizaciones nuevas.");
			}

		} catch (MalformedURLException ex) {
			YakiToOutput.error("La URL de actualizaciones (" + defaultPathFile
					+ ") es incorrecta.");
		} catch (IOException e) {
			YakiToOutput.error("No se pudo abrir la conexión a "
					+ defaultPathFile);
		}
		return updates;
	}

	/**
	 * Realiza las actualizaciones
	 */
	public void update() {

		HttpURLConnection urlc;
		InputStream inStream;
		OutputStream outStream;

		try {
			URL url = new URL(defaultPathFile);
			File destiny = new File(config.getJarLocationPath() + "/update.zip");
			urlc = (HttpURLConnection) url.openConnection();
			inStream = urlc.getInputStream();
			outStream = new FileOutputStream(destiny);
			simpleCopyWithoutClosing(inStream, outStream);
			urlc.disconnect();
			inStream.close();
			outStream.close();
			YakiToOutput.message("Las actualizaciones se han descargado "
					+ "correctamente. Instalando actualizaciones...");
			// TODO: Instalacion de actualizaciones
			// Descomprimimos el fichero de actualizaciones
			YakiToFileSystem.unzipFile(destiny);

			// Eliminamos el ZIP
			// destiny.delete();
			// Establecemos la fecha de la ultima actualizacion
			config.setLastKnownUpdate(lastUpdate);
		} catch (FileNotFoundException e) {
			YakiToOutput.error("No se encontro un fichero remoto de "
					+ "actualizaciones. Trate de actualizar de forma"
					+ " manual más tarde.");
			e.printStackTrace();
		} catch (IOException e) {
			YakiToOutput
					.error("Error de entrada/salida mientras se descargaban "
							+ "o descomprimian "
							+ "las actualizaciones. Trate de actualizar de "
							+ "forma manual más tarde.");
		}
	}

	

	private void simpleCopyWithoutClosing(InputStream in, OutputStream out)
			throws IOException {
		byte[] buf = new byte[1024];
		int len, len2 = 0, retardador = 0;
		long readed = 0;
		Stopwatch c = new Stopwatch(), aux = new Stopwatch();
		c.startClock();
		aux.startClock();
		while ((len = in.read(buf)) > 0) {
			if (retardador % 20 != 0) {
				len2 += len;
			}
			retardador++;
			out.write(buf, 0, len);
			readed += len;
			if (retardador % 20 == 0) {
				c.stopClock();
				aux.stopClock();
				aux.startClock();
				len2 = 0;
			}
		}
	}

}
