package net.yakiboo.yakito.utils;

import java.util.Calendar;

/**
 * Clase para el cronometrado. Un objeto <code>Cronometro</code> calcula el
 * tiempo transcurrido entre dos instantes de tiempo: el instante en que se
 * llama al metodo <code>startClock()</code> y el instante en que se llama al
 * método <code>stopClock()</code>.
 * 
 * @author Sergio Garcia Mondaray
 */
public class Stopwatch {

	long inicio;
	long fin;
	Calendar c;

	/**
	 * Constructor del cronometro
	 */
	public Stopwatch() {
		inicio = 0;
		fin = 0;
	}

	/**
	 * Establece el punto de inicio de cronometrado en el instante actual.
	 */
	public void startClock() {
		inicio = fin = getActualTime();
	}

	/**
	 * Establece el punto de fin de cronometrado en el instante actual.
	 */
	public void stopClock() {
		fin = getActualTime();
	}

	private long getActualTime() {
		c = Calendar.getInstance();
		return c.getTimeInMillis();
	}

	/**
	 * Calcula el tiempo transcurrido entre las ultimas llamadas a
	 * <code>startClock()</code> y <code>stopClock()</code> en milisegundos.
	 * 
	 * @return tiempo cronometrado
	 */
	public long getResult() {
		return fin - inicio;
	}

	/**
	 * Igual que <code>getResult()</code>, solo que el resultado se devuelve en
	 * segundos.
	 * 
	 * @return tiempo cronometrado
	 */
	public double getResultInSeconds() {
		return getResult() / 1000.0;
	}

}
